// -*- C++ -*-
/*!
 * @file  RTC_NineDOF_Ardiuno.h
 * @brief 9軸センサをつないだArdiuno（または何らかのマイコン）からセンサデータ取得。機体位置姿勢推定
 * @date  $Date$
 *
 * @author AIZUK
 * kohno@aizuk.jp
 *
 * $Id$
 */

#ifndef RTC_NINEDOF_ARDIUNO_H
#define RTC_NINEDOF_ARDIUNO_H

#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="port_stub_h">
// </rtc-template>

using namespace RTC;

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <curses.h>
#include <sys/time.h>
#include <time.h>
#include <fstream>
#include <math.h>

/*!
 * @class RTC_NineDOF_Ardiuno
 * @brief 9軸センサをつないだArdiuno（または何らかのマイコン）からセンサデータ取得。機体位置姿勢推定
 *
 * ９軸センサデータを取得するArdiunoと接続
 * 加速度・角速度・磁束強度をシリアル通信で取得
 * カルマンフィルタ（または単純な積分）により位置姿勢推定したものを
 * 出力
 * 姿勢推定
 * カルマンフィルタの観測値のデフォルト
 * roll,pitch:加速度による推定
 * yaw:地磁気による推定
 *
 * 入力
 * timed pose3D
 * data.x ,datay,data.z:
 * data.p ,datar,data.y:姿勢観測値（必須でない）
 * 出力
 * timed pose3D
 * data.x ,datay,data.z:機体位置推定値
 * data.p ,datar,data.y:機体姿勢推定値
 *
 * カルマンフィルタ
 * メディアンフィルタ
 *
 * Sebastian Thrun, Wolfram Burgard, Dieter Fox(著) 上田隆一
 * (訳)(2015)『確率ロボティクス』 マイナビ.
 * 足立修一・丸田一郎 （2012）　『カルマンフィルタの基礎』
 * 東京電機大学出版局
 *
 */
class RTC_NineDOF_Ardiuno
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  RTC_NineDOF_Ardiuno(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~RTC_NineDOF_Ardiuno();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /***
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <curses.h>
#include <sys/time.h>
#include <time.h>
#include <fstream>
#include <math.h>

   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);
   
   
  /***
   *
   * 
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t Connect_serial();
   
  /***
   *
   * 
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t Kal_Cal_YPR();

  /***
   *
   * 
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t Kal_Cal_XYZ();
  /***
   *
   * 
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t cal_YPR(double* ACC_,double* GYR_,double* MAG_,double Y_0,double Y_1,double Y_2);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // Configuration variable declaration
  // <rtc-template block="config_declare">
  /*!
   * カルマンフィルタON　　　：１
   * カルマンフィルタOFF　　           ：0
   * - Name:  Kalman_ON_OFF
   * - DefaultValue: 1
   */
  int m_Kalman_ON_OFF;
  /*!
   * デバイスポート名
   * - Name:  Port
   * - DefaultValue: /dev/ttyUSB3
   */
  std::string m_Port;
  /*!
   * デバイス通信速度
   * - Name:  BAUDRATE
   * - DefaultValue: B115200
   */
  std::string m_BAUDRATE;
  /*!
   * メディアンフィルタON/OFF　(データノイズ除去)
   * メディアンフィルタON　　　：１
   * メディアンフィルタOFF　　           ：0
   * - Name:  Median_ON_OFF
   * - DefaultValue: 1
   */
  int m_Median_ON_OFF;
  /*!
   * 他のセンサデータをカルマンフィルタ観測値として使用(Pose_In,Or
   * ientation_In)
   * データはInport [roll,pitch,yaw]
   * Orientation_FIX :機能
   * 0：OFF
   * １：yawのみ
   * ２：pitchのみ
   * ３：rollのみ
   * ４：yaw,pitch
   * ５：yaw,roll
   * ６：roll,pitch
   * ６：yaw,roll,pitch
   * - Name:  Orientation_FIX
   * - DefaultValue: 0
   */
  int m_Orientation_FIX;
  /*!
   * 他のセンサデータをカルマンフィルタ観測値として使用
   * (Pose_In,Position_In)
   * [x,y,z]
   * Position_FIX :機能
   * 0：OFF
   * １：xのみ
   * ２：yのみ
   * ３：Zのみ
   * ４：x,y
   * ５：x,z
   * ６：y,z
   * ６：x,y,z
   * - Name:  Position_FIX
   * - DefaultValue: 0
   */
  int m_Position_FIX;
  /*!
   * メディアンフィルタで扱うデータ数
   * - Name:  Median_NUM
   * - DefaultValue: 3
   */
  int m_Median_NUM;
  /*!
   * Yawオフセット
   * - Name:  Yaw_OFFSET
   * - DefaultValue: 0
   */
  double m_Yaw_OFFSET;
  /*!
   * Pitchオフセット
   * - Name:  Pitch_OFFSET
   * - DefaultValue: 0
   */
  double m_Pitch_OFFSET;
  /*!
   * Rollオフセット
   * - Name:  Roll_OFFSET
   * - DefaultValue: 0
   */
  double m_Roll_OFFSET;

  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  RTC::TimedPose3D m_Pose_In;
  /*!
   * 期待位置姿勢観測値[x,y,z,roll,pitch,yaw]
   * - Unit: [m] [rad]
   */
  InPort<RTC::TimedPose3D> m_Pose_InIn;
  RTC::TimedPoint3D m_Position_In;
  /*!
   * 機体位位置観測値[x,y,z]
   * - Unit: [m]
   */
  InPort<RTC::TimedPoint3D> m_Position_InIn;
  RTC::TimedOrientation3D m_Orientation_In;
  /*!
   * 機体姿勢観測値[roll,pitch,yaw]
   * - Unit: [rad]
   */
  InPort<RTC::TimedOrientation3D> m_Orientation_InIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  RTC::TimedPose3D m_Pose_Out;
  /*!
   * 期待位置姿勢[x,y,z,roll,pitch,yaw]
   * - Unit: [m] [rad]
   */
  OutPort<RTC::TimedPose3D> m_Pose_OutOut;
  RTC::TimedPoint3D m_Position_Out;
  /*!
   * 機体位置[x,y,z]
   * - Unit: [m]
   */
  OutPort<RTC::TimedPoint3D> m_Position_OutOut;
  RTC::TimedOrientation3D m_Orientation_Out; 
  /*!
   * 姿勢[roll,pitch,yaw]
   * - Unit: [rad]
   */
  OutPort<RTC::TimedOrientation3D> m_Orientation_OutOut;
  RTC::TimedAcceleration3D m_Acc;
  /*!
   * 加速度データ[x,y,z]
   * - Unit: [m/sec^2]
   */
  OutPort<RTC::TimedAcceleration3D> m_AccOut;
  RTC::TimedAngularAcceleration3D m_Gyro;
  /*!
   * ジャイロデータ[x,y,z]
   * - Unit: [rad/sec]
   */
  OutPort<RTC::TimedAngularAcceleration3D> m_GyroOut;
  RTC::TimedVector3D m_Mag;
  /*!
   * 地磁気データ[x,y,z]
   */
  OutPort<RTC::TimedVector3D> m_MagOut;
  RTC::TimedDouble m_Tmp;
  /*!
   * 温度
   * - Unit: [℃]
   */
  OutPort<RTC::TimedDouble> m_TmpOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  // <rtc-template block="private_attribute">
  
  // </rtc-template>

  // <rtc-template block="private_operation">
  
  // </rtc-template>

};


extern "C"
{
  DLL_EXPORT void RTC_NineDOF_ArdiunoInit(RTC::Manager* manager);
};

#endif // RTC_NINEDOF_ARDIUNO_H
